package com.sun.grizzly.jruby.rack;


import com.sun.grizzly.http.SelectorThread;
import com.sun.grizzly.jruby.RailsAdapter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.logging.Logger;

/**
 * @author Jacob Kessler
 * Attempts to determine the RackApplicationFactory most suited to handle
 * the given application directory
 */
public class RackApplicationChooser {
    public static RackAdapter getFactory(String appPath, RailsAdapter adapter) {
        Logger theLog = adapter.getLogger();
        String appType = System.getProperty("jruby.applicationType");
        if (appType != null) {
            // User has specified a file to run. For now, assume that it is single-threaded
            RackApplicationFactory myFactory = new GenericApplicationFactory(appType, adapter);
            return new SingleThreadedRackAdapter(myFactory, adapter);
        }
            // Sinatra may not have a rake file. Check if we were given a .rb file
            if (appPath.endsWith(".rb")) {
                File appFile = new File(appPath);
                String app = inhale(appFile, theLog);
                if (app.contains("require 'sinatra'")) {
                    RackApplicationFactory myFactory = new SinatraApplicationFactory(adapter);
                    theLog.info("Detected Sinatra application");
                    // Sinatra is thread-safe
                    return new MultiThreadedRackAdapter(myFactory, adapter);
                }
            }
            theLog.fine("Path is " + appPath + ", trying to open " + appPath+"Rakefile");
            File rakeFile = new File(appPath, "Rakefile");
            if (rakeFile.exists()) {
            String rake = inhale(rakeFile, theLog);

                // rails
                if (rake.contains("require 'tasks/rails'")) {
                    RackApplicationFactory myFactory = new RailsApplicationFactory(adapter);
                    theLog.info("Detected Rails application");
                    // Check rails thread safety
                    boolean multithreaded = false;
                    String environment = "";
                    if (System.getProperty("rails.env") == null || "development".compareTo(System.getProperty("rails.env")) == 0) {
                         // Grab the development environment file
                         environment = inhale(new File(appPath+"/config/environments/development.rb"), theLog);
                    } else if ("production".compareTo(System.getProperty("rails.env")) == 0) {
                        // if we are in production mode
                        environment = inhale(new File(appPath+"/config/environments/production.rb"), theLog);
                    } else if ("testing".compareTo(System.getProperty("rails.env")) == 0) {
                        environment = inhale(new File(appPath+"/config/environments/testing.rb"), theLog);
                    }
                    if (environment.matches("(?s)(?m).*^[^#]*config\\.threadsafe!.*")) { // If there is a non-commented line with config.threadsafe! in it
                            multithreaded = true;
                        }
                    if (multithreaded) {
                        theLog.info("config.threadsafe sighted! All hands prepare for multithread mode!");
                        return new MultiThreadedRackAdapter(myFactory, adapter);
                    } else {
                        theLog.info("Rails not in thread-safe mode, starting in single-thread mode");
                        return new SingleThreadedRackAdapter(myFactory, adapter);
                    }
                }
                // merb
                if (rake.contains("require 'merb-core'")) {
                    RackApplicationFactory myFactory = new MerbApplicationFactory(adapter);
                    theLog.info("Detected merb application");
                    // Merb is thread-safe
                    return new MultiThreadedRackAdapter(myFactory, adapter);
                }
                // Other frameworks go here
            } else {
                theLog.fine("Rake file not found, checking for non-rake frameworks");
                // Check for .rb files
                File appDir = new File(appPath);
                if (appDir.isDirectory()) {
                    File[] files = appDir.listFiles();
                    for (File file : files) {
                        if (file.getAbsolutePath().endsWith(".rb")) {
                            // Check for require. This is expensive and we don't want to.
                            String rb = inhale(file, theLog);
                            if (rb.contains("require 'sinatra'") || rb.contains("require \"sinatra\"")) {
                                RackApplicationFactory myFactory = new SinatraApplicationFactory(adapter);
                                // Sinatra is still thread-safe
                                return new MultiThreadedRackAdapter(myFactory, adapter);
                            }

                        }
                    }
                }
            }
        // No framework found
        SelectorThread.logger().severe("Framework autodetection failed! Please set -Djruby.applicationType to a script that will start your framework");
        throw new IllegalStateException("No framework to start!");
    }

    private static String inhale (File f, Logger theLog) {
        String app = "";
        if (f.exists()) {
            try {
                BufferedReader in = new BufferedReader(new FileReader(f));
                String s;
                while ((s = in.readLine()) != null) {
                    // read the file
                    app = app + s + "\n";
                }
            } catch (IOException e) {
                theLog.severe("IO Exception determining framework for " + f.getAbsolutePath() + ": " + e);
            }
        }
        return app;
    }

}
